//
//*****************************************************************************
//*                                                                           *
//* COPYRIGHT (c) Maelzel, Inc.,                                              *
//*                                                                           *
//*          Redistribution and use of this code and the algorithms herein    *
//*          contained, with or without modification, is permitted provided   *
//*          that the following conditions are met:                           *
//*          * Redistribution of this code and the algorithms herein          *
//*            contained must retain the above copyright notice, this list    *
//*            of conditions, and the following disclaimer.                   *
//*          * Any use of this code and the algorithms herein contained as    *
//*            text or in binary form must attribute, in a reasonable         *
//*            manner, the provenance and authorship of this code as          *
//*            shown in the copyright above.                                  *
//*                                                                           *
//* DISCLAIMER This code and the algorithms herein contained are provided     *
//*            by the developer ''as is'' without any expressed or implied    *
//*            warranties, including, but not limited to, fitness for any     *
//*            particular purpose or any consequential damage that may        *
//*            occur as a consequence of use or association with this code    *
//*            or the algorithms herein contained.                            *
//*                                                                           *
//* TOP      Clans.java                                                       *
//* CLASS    ModelConstraints.java. (contains model constraint data)          *
//* AUTHOR   Ovi Chris Rouly                                                  *
//*                                                                           *
//* DATE     06/06/2015                                                       *
//* REVISION X.0.1.3                                                          *
//* BYTES    00,000                                                           *
//* TIME     21:24                                                            *
//* PURPOSE  This object contains model parameters.                           *
//* METHOD   Used as a super class for inheritance of data.                   *
//* DETAILS  A Java document exists that outlines class operation.  Other     *
//*          documentation in the form of programmer notes apply.             *
//* ENVIRON  Developed on Intel Core-2 Quad Q8200 2.33 GHz CPU, Nvidia-based  *
//*          PCI-X graphics acceleration.  The OS was MS Windows 7 Ultimate.  *
//*          Java IDE was JGrasp version 2.0.0_08.  The Java compiler was     *
//*          version 1.8.0.  Verification tests were run in Java JRE          *
//*          build 1.8.0-b132 with client VM build 14.3-b01.                  *
//*                                                                           *
//*****************************************************************************
//

package root;                                         // sub-directory storage.

import java.io.File;                                          // Java language.

//
//*****************************************************************************
// ModelConstraints


public interface ModelConstraints
{

  //
  //***************************************************************************
  // GLOBAL constants.

  final public static boolean USE_FIXED = false;// RNG seed is fixed or random.
  final public static long CALIB_SEED = 0;  // a fixed seed.
  final public static long RANDOM_SEED = System.nanoTime();  // fast time seed.

  // RUN_TIME must be set FALSE in order to use the two DIAGNOSTIC modes.
  final public boolean RUN_TIME_DISPLAY = true; //true=normal false=DIAGNOSTIC.
  final public boolean FORAGING_DISPLAY = true; //true=forage false=waterBasin.

  // Installation, operating system, and hardware RNG dependencies.
  final public static String TRUE_RNG_PORT = "COM5";
  final public static String BIN_FOLDER = "root" + File.separator;
  final public static String DATA_FOLDER = "data" + File.separator;
  final public static String MAP_FOLDER = "maps" + File.separator;

  final public static int WATER = 0;    // terrestrial feature specific to Sim.
  final public static int LO_LAND = 1;  // terrestrial feature specific to Sim.
  final public static int HI_LAND = 2;  // terrestrial feature specific to Sim.

  final public static int YEAR = 364;                        // a year of days.
  final public static float FAT_AS_CALORIE = 3500.0F;      // 1 lb. conversion.

  // --------------------------------------------------------------------------

  final public boolean FIXED_GRID = true;        // seed-group on a fixed grid.

  // --------------------------------------------------------------------------

  final public boolean USE_ALTRUISM = true;  // can engage altruistic behavior.

  // --------------------------------------------------------------------------

  final public static int DISCRETE_FACTORS_GENE =   0; // gene 0 on chromosome.
  final public static int DRAUGHT_TOLERANCE_GENE =  1; // gene 1 on chromosome.
  final public static int ROBUST_METABOLISM_GENE =  2; // gene 2 on chromosome.
  final public static int SOCIAL_ALTRUISM_GENE   =  3; // gene 3 on chromosome.
  final public static int TERRITORY_DEFENSE_GENE =  4; // gene 4 on chromosome.
  final public static int PAIR_BND_ALLIANCE_GENE =  5; // gene 5 on chromosome.
  final public static int EMIT_FERTIL_SCENT_GENE =  6; // gene 6 on chromosome.
  final public static int PHILOPAT_DISPERSE_GENE =  7; // gene 7 on chromosome.
  final public static int RESERVED_GENE_8        =  8; // gene 8 on chromosome.
  final public static int RESERVED_GENE_9        =  9; // gene 9 on chromosome.
  final public static int TEMPERTURE_SETPNT_GENE = 10; // gene A on chromosome.

  // --------------------------------------------------------------------------

  final public static int MAXIMUM_GROUP_SIZE = 100;    // max nr in seed group.
  final public static int NUMBER_OF_GROUPS = 2;    // max nr of hominid groups.

  // --------------------------------------------------------------------------

  // produced 8863 years
  final public int WATER_DAYS = 2;  // max allowed consecutive epochs in water.

  // --------------------------------------------------------------------------

  // produced 8863 years
  final public int PREY_ATTITUDE_CHANGE_THRESHOLD = 0364;    //  1  day = 0001.
                                                       //  1 week equiv = 0007.
                                                       //  1 year equiv = 0364.
                                                       //  5 year equiv = 1820.
                                                       // 10 year equiv = 3640.
                                                       // 15 year equiv = 5460.

  // --------------------------------------------------------------------------

//final public float CHANCE_OF_PREDATION_HI = 0.0001600F;// dy, 5.50% yr, Boes.
  final public float CHANCE_OF_PREDATION_HI = 0.0000140F;// dy, 0.50% yr, esti.
  final public float CHANCE_OF_PREDATION_LO = 0.0000001F;// dy, 0.04% yr, Will.
  // Williams, J, et. al. (2008), Am J Prim, vol. 70, pp. 776-777. "none seen"
  // Boesch, C. (1991), Behaviour, vol. 117, nr. 3/4, pp. 220-242 (p.229-230).

  // #/y = 1 - ( 1 - p ) ^ 364; example: 0.055/y = 1 - ( 1 - 0.00016 ) ^ 364

  // --------------------------------------------------------------------------

  // produced 8863 years
  final public static float INFANTICIDE_MORT = 0.000055F; //daily, 2.0% annual.
  final public static float DISEASE_THRESHOLD = 0.000055F;//daily, 2.0% annual.

  // --------------------------------------------------------------------------
  // infant weaning must be complete between 4 and 6 years of age, inclusive.

  // produced 8863 years
  final public static int RANGE_OF_EARLY_WEANING = 3;    // 0-2 by factory RNG.
  final public static int MIN_WEANING_AGE = 4;   // minimum oldest when weaned.

  // --------------------------------------------------------------------------

  // produced 8863 years
  final float MIN_WATER = 1.0F;   // minimum gals of water each basin will get.
  final float RANDOM_WATER = 3.50F;    // some basins get up to this much more.
  final float RATE_OF_LOWLAND_WATER_BASINS = 0.5F; // prcnt cells water basins.
  final float RATE_OF_HIGHLAND_WATER_BASINS = 0.3F;// prcnt cells water basins.

  // --------------------------------------------------------------------------

  // produced 8863 years
  //final public float FLORA_SUCCESS_BIAS = 0.75F; // min seasonal foragable prd.
  final public float FLORA_SUCCESS_BIAS = 0.50F; // min seasonal foragable prd.
  final public float FAUNA_SUCCESS_BIAS = 0.50F; // min seasonal foragable prd.
  final public float INITIAL_PRODUCTIVITY = 99.0F; // initial prcnt production.
  final public float FORAGING_COST = 1.0F; // prcnt cost to terrain if foraged.
  final public float FORAGING_RCVRY_RATE = 0.50F;  // prcnt terrain rcvry rate.

  // --------------------------------------------------------------------------

  // produced 8863 years
  // equivalent to eating 100% of a 2700 kCal diet composed entirely of tubers.
  final public float DAILY_MEALS = 3.48F;//"multiplier" best case situation.

  // --------------------------------------------------------------------------

  // produced 8863 years
  final public static int AFFECT_WORD_WIDTH = 8;  // Java can support up to 30.
  final public static int GROUPED_DRIVES = 3;   // nr of highest priorty group.

  // --------------------------------------------------------------------------

  // produced 8863 years
  final public int FEAR_STEP = 2;    // n X 10 meter movement distance per day.
  final public int WATER_STEP = 15;  // n X 10 meter movement distance per day.
  final public int FOOD_STEP = 15;   // n X 10 meter movement distance per day.
  final public int GROUP_STEP = 30;  // n X 10 meter movement distance per day.

  // --------------------------------------------------------------------------
  // From Biology.java (24 Jan 2015)
  // MALE: (adult maximum requirements, calculated)
  // Water = 0.750F;
  // Food  = 0.700F;
  //
  // FEMALE: (adult maximum requirements, calculated - w/reproductive loading)
  // Water = 0.975F;
  // Food  = 0.465F;

  // produced 8863 years
  final public float WATER_START_AMOUNT = 0.7F;  // water amount to start with.
  final public float WATER_CARRYING_LIMIT = 1.0F; // only hand-to-mouth (gals).

  final public float FOOD_START_AMOUNT = 0.7F;    // food amount to start with.
  final public float FOOD_CARRYING_LIMIT_F = 0.7F; // only hand to mouth (lbs).
  final public float FOOD_CARRYING_LIMIT_M = 0.9F; // only hand to mouth (lbs).

  // --------------------------------------------------------------------------

}
