//
//*****************************************************************************
//*                                                                           *
//* COPYRIGHT (c) Maelzel, Inc.,                                              *
//*                                                                           *
//*          Redistribution and use of this code and the algorithms herein    *
//*          contained, with or without modification, is permitted provided   *
//*          that the following conditions are met:                           *
//*          * Redistribution of this code and the algorithms herein          *
//*            contained must retain the above copyright notice, this list    *
//*            of conditions, and the following disclaimer.                   *
//*          * Any use of this code and the algorithms herein contained as    *
//*            text or in binary form must attribute, in a reasonable         *
//*            manner, the provenance and authorship of this code as          *
//*            shown in the copyright above.                                  *
//*                                                                           *
//* DISCLAIMER This code and the algorithms herein contained are provided     *
//*            by the developer ''as is'' without any expressed or implied    *
//*            warranties, including, but not limited to, fitness for any     *
//*            particular purpose or any consequential damage that may        *
//*            occur as a consequence of use or association with this code    *
//*            or the algorithms herein contained.                            *
//*                                                                           *
//* TOP      Clans.java                                                       *
//* CLASS    RGB.java (Red, Green, Blue.)                                     *
//* AUTHOR   Ovi Chris Rouly                                                  *
//*                                                                           *
//* DATE     06/06/2015                                                       *
//* REVISION X.0.1.2                                                          *
//* BYTES    00,000                                                           *
//* TIME     15:55                                                            *
//* PURPOSE  This object creates RGB, 24-bit color integers.                  *
//* METHOD   Each instance of a class is a stand-alone code object.           *
//* DETAILS  A Java document exists that outlines class operation.  Other     *
//*          documentation in the form of programmer notes apply.             *
//* ENVIRON  Developed on Intel Core-2 Quad Q8200 2.33 GHz CPU, Nvidia-based  *
//*          PCI-X graphics acceleration.  The OS was MS Windows 7 Ultimate.  *
//*          Java IDE was JGrasp version 2.0.0_08.  The Java compiler was     *
//*          version 1.8.0.  Verification tests were run in Java JRE          *
//*          build 1.8.0-b132 with client VM build 14.3-b01.                  *
//*                                                                           *
//*****************************************************************************
//

package root.gui;                                            // home directory.

//*****************************************************************************
//

public class RGB
{

  //
  //***************************************************************************
  // Class constants, properties, and objects.

  // Constants.
  private final static int ALPHA_BASE = 16777216;
  private final static int RED_BASE = 65536;
  private final static int GREEN_BASE = 256;
  private final static int BLUE_BASE = 1;

  private static int RED = 0;             // self explanatory, left as mutable.
  private static int GREEN = 0;                                       // ditto.
  private static int BLUE =  0;                                       // ditto.
  private static int BROWN = 0;                                       // ditto.
  private static int ORANGE = 0;                                      // ditto.
  private static int PURPLE = 0;                                      // ditto.
  private static int WHITE = 0;                                       // ditto.
  private static int BLACK = 0;                                       // ditto.

  //
  //***************************************************************************
  // Variables.
  private int red = 0;
  private int green = 0;
  private int blue =  0;
  private int brown = 0;
  private int orange = 0;
  private int purple = 0;
  private int white = 0;
  private int black = 0;

  // Objects.

  //***************************************************************************
  /** This constructor instantiates an instance of the RGB object.*/

  public RGB()
  {

    RED =    Red(255);                                 // self explanatory.
    GREEN =  Green(255);                               // self explanatory.
    BLUE =   Blue(255);                                // self explanatory.
    BROWN =  Red(128) + Green(64);                     // self explanatory.
    ORANGE = Red(255) + Green(128);                    // self explanatory.
    PURPLE = Red(128) + Blue(128);                     // self explanatory.
    WHITE =  RED + GREEN + BLUE;                       // self explanatory.
    BLACK =  0;                                        // self explanatory.

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns the RGB integer value for blue. */

  public int Blue(int blueColor)
  {

    // clamp to limits.
    if (blueColor > 255)
      blueColor = 255;
    if (blueColor < 0)
      blueColor = 0;

    return BLUE_BASE * blueColor;

  } // End method.

  //***************************************************************************

  //***************************************************************************
  /** This method returns the RGB integer value for green.*/

  public int Green(int greenColor)
  {

    // clamp to limits.
    if (greenColor > 255)
      greenColor = 255;
    if (greenColor < 0)
      greenColor = 0;

    return GREEN_BASE * greenColor;

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns the RGB integer value for alpha.*/

  public int Alpha(int theAlpha)
  {

    if (theAlpha > 255)
      theAlpha = 255;
    if (theAlpha < 0)
      theAlpha = 0;

    return ALPHA_BASE * theAlpha;

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method disassembles an RGB color into it component parts.  The
      results, i.e., red, green, blue, are available after calling this
      method by accessor.  For example: getRedComponent().*/

  public void colorDisassembler(int mixedColor)
  {

    int testColor = Math.abs(mixedColor);

    int redPart = 0;
    int greenPart = 0;
    int bluePart = 0;

    int remainder = 0;

    // collect red component.
    redPart = (int) (testColor / RED_BASE);
    remainder = testColor - (redPart * RED_BASE);
    red = ( (remainder >= 0) ? redPart : 0 );

    // collect green component.
    greenPart = (int) (remainder / GREEN_BASE);
    remainder = remainder - (greenPart * GREEN_BASE);
    green = ( (remainder >= 0) ? greenPart : 0 );

    // collect blue component.
    blue = remainder;

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns the blue component (an integer) found after running
      rgbColorDissassembler on a mixed RGB color.*/

  public int getBlueComponent()
  {

    return blue;

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns the green component (an integer) found after running
      rgbColorDissassembler on a mixed RGB color.*/

  public int getGreenComponent()
  {

    return green;

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns the red component (an integer) found after running
      rgbColorDissassembler on a mixed RGB color.*/

  public int getRedComponent()
  {

    return red;

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns the RGB integer value for a shade of gray. */

  public int Gray(int intensity)
  {

   return Red(intensity) + Green(intensity) + Blue(intensity);

  } // End method.

  //***************************************************************************

  //***************************************************************************
  /** This method returns the RGB integer value for the hue requested. */

  public int Hue(int redIntensity, int greenIntensity, int blueIntensity)
  {

   return Red(redIntensity) + Green(greenIntensity) + Blue(blueIntensity);

  } // End method.

  //***************************************************************************

  //***************************************************************************
  /** This method returns the RGB integer value for red.*/

  public int Red(int redColor)
  {

    // clamp to limits.
    if (redColor > 255)
      redColor = 255;
    if (redColor < 0)
      redColor = 0;

    return RED_BASE * redColor;

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method does something.*/

  public void template()
  {
    //add code here.

  } // End method.
  //***************************************************************************

}
