//
//*****************************************************************************
//*                                                                           *
//* COPYRIGHT (c) Maelzel, Inc.,                                              *
//*                                                                           *
//*          Redistribution and use of this code and the algorithms herein    *
//*          contained, with or without modification, is permitted provided   *
//*          that the following conditions are met:                           *
//*          * Redistribution of this code and the algorithms herein          *
//*            contained must retain the above copyright notice, this list    *
//*            of conditions, and the following disclaimer.                   *
//*          * Any use of this code and the algorithms herein contained as    *
//*            text or in binary form must attribute, in a reasonable         *
//*            manner, the provenance and authorship of this code as          *
//*            shown in the copyright above.                                  *
//*                                                                           *
//* DISCLAIMER This code and the algorithms herein contained are provided     *
//*            by the developer ''as is'' without any expressed or implied    *
//*            warranties, including, but not limited to, fitness for any     *
//*            particular purpose or any consequential damage that may        *
//*            occur as a consequence of use or association with this code    *
//*            or the algorithms herein contained.                            *
//*                                                                           *
//* CLASS    Clans.java. (Model of something)                                 *
//* CLASS    TrueRNG.java. (True Random Number Generator)                     *
//* AUTHOR   Ovi Chris Rouly                                                  *
//*                                                                           *
//* DATE     09/27/2014                                                       *
//* REVISION X.0.0.0                                                          *
//* BYTES    nn,nnn                                                           *
//* TIME     00:00                                                            *
//* PURPOSE  To do something.                                                 *
//* METHOD   By means of software simulation, model proto-humans.             *
//* DETAILS  Based on the resources of the usb4java website and many others.  *
//* ENVIRON  Developed on Intel Pentium D 3.4GHz CPU, 4GB RAM, Nvidia-based   *
//*          PCI-X graphics acceleration.  The OS was MS XP Home SP3. Java    *
//*          IDE was JGrasp version 1.8.8_10.  The Java compiler was          *
//*          version 1.6.0_17.  Verification tests were run in Java JRE       *
//*          build 1.6.0_17-b04 with client VM build 14.3-b01.                *
//* COMPILE  javac -g -verbose -deprecation Model.java                        *
//* RUN AS   java -client -Xms1500m -Xmx1500m -Xss1m GenderDominance -cp      *
//*                                                                           *
//*****************************************************************************
//

package rng;                                                 // home directory.

import java.util.List;
import javax.usb.UsbConfiguration;
import javax.usb.UsbDevice;
import javax.usb.UsbDeviceDescriptor;
import javax.usb.UsbEndpoint;
import javax.usb.UsbException;
import javax.usb.UsbHostManager;
import javax.usb.UsbHub;
import javax.usb.UsbInterface;
import javax.usb.UsbPipe;
import javax.usb.UsbPort;
import javax.usb.UsbServices;
import org.usb4java.Context;
import org.usb4java.Device;
import org.usb4java.DeviceDescriptor;
import org.usb4java.DeviceHandle;
import org.usb4java.LibUsb;
import org.usb4java.LibUsbException;

//
//*****************************************************************************
// TrueRNG

public class TrueRNG
{

  //
  //***************************************************************************
  // Class constants, variables, and objects.

  // Constants.
  private final static byte INTFACE_RST = (byte) 0xff; // unproven bogus value.
  private final static byte INTERFACE_1 = (byte) 0x00;      // end point #1 in.
  private final static byte INTERFACE_2 = (byte) 0x01; //end point #2 in & out.

  private final static byte DATA_OUT_END_PNT = (byte) 0x02;  // data interface.
  private final static byte CNTL_IN_END_PNT = (byte) 0x81; //control interface.
  private final static byte DATA_IN_END_PNT = (byte) 0x82;   // data interface.

  private final static short VENDOR_ID = (short) 0x04d8;
  private final static short PRODUCT_ID = (short) 0xf5fe;

  // Variables.

  // Objects.
  private static Context context = null;
  private static Device device = null;
  private static DeviceDescriptor descriptor = null;
  private static DeviceHandle handle = null;
  private static UsbConfiguration uConfiguration = null;
  private static UsbDevice uDevice = null;
  private static UsbHub hub = null;
  private static UsbInterface iface = null;
  private static UsbServices services = null; 

  //***************************************************************************
  /** This is a class constructor.*/
  
  public TrueRNG ()
  {
  }
  //***************************************************************************
  
  //***************************************************************************
  /** This method does nothing.*/

  public static void dumpDevices () 
  { 

    try
    {
      // Get the USB services and dump information about them 
      services = UsbHostManager.getUsbServices(); 
      System.out.println("USB Service Implementation: " + 
                         services.getImpDescription()); 
      System.out.println("Implementation version: " +
                         services.getImpVersion()); 
      System.out.println("Service API version: " +
                         services.getApiVersion()); 
      System.out.println(); 

      // Dump the root USB hub 
      dumpDevicesHelper(services.getRootUsbHub()); 

    }
    catch (UsbException ue)
    {
      System.out.println("ERROR: cannot dump connected devices list.");
    }

  } 
  //***************************************************************************
  
  //***************************************************************************
  /** This method does nothing.*/

  private static void dumpDevicesHelper (UsbDevice device) 
  { 
    // Dump information about the device itself 
    System.out.println(device); 
    final UsbPort port = device.getParentUsbPort(); 
    if (port != null) 
    { 
      System.out.println("Connected to port: " + port.getPortNumber()); 
      System.out.println("Parent: " + port.getUsbHub());
      hub = port.getUsbHub();
    } 

    // Dump device descriptor 
    System.out.println(device.getUsbDeviceDescriptor()); 

    // Process all configurations 
    for (UsbConfiguration configuration: (List<UsbConfiguration>) device 
        .getUsbConfigurations()) 
    { 
      // Dump configuration descriptor 
      System.out.println(configuration.getUsbConfigurationDescriptor()); 

      // Process all interfaces 
      for (UsbInterface iface: (List<UsbInterface>) configuration 
          .getUsbInterfaces()) 
      { 
        // Dump the interface descriptor 
        System.out.println(iface.getUsbInterfaceDescriptor()); 


        // Process all endpoints 
        for (UsbEndpoint endpoint: (List<UsbEndpoint>) iface 
            .getUsbEndpoints()) 
        { 
          // Dump the endpoint descriptor 
          System.out.println(endpoint.getUsbEndpointDescriptor()); 
        } 
      } 
    } 

    System.out.println(); 

    // Dump child devices if device is a hub 
    if (device.isUsbHub()) 
    { 
      final UsbHub hub = (UsbHub) device; 
      for (UsbDevice child: (List<UsbDevice>) hub.getAttachedUsbDevices()) 
      { 
          dumpDevicesHelper(child); 
      } 
    }

  } 
  //***************************************************************************

  //***************************************************************************
  /** This method does nothing.*/

  private static class EventHandlingThread extends Thread
  {

    private volatile boolean abort;

    public void abort()
    {
      this.abort = true;
    }

    @ Override
    public void run()
    {
      while (!this.abort)
      {
        int result = LibUsb.handleEventsTimeout(null, 1000);
        if (result != LibUsb.SUCCESS)
          throw new LibUsbException("Unable to handle events", result);
      }
    }

  }
  //***************************************************************************

  //***************************************************************************
  /** This method does nothing.*/

  public static UsbDevice findDevice(UsbHub hub,
                                     short vendorId, short productId)
  {
    for (UsbDevice fDevice : (List<UsbDevice>) hub.getAttachedUsbDevices())
    {
      UsbDeviceDescriptor desc = fDevice.getUsbDeviceDescriptor();
      if (desc.idVendor() == vendorId && desc.idProduct() == productId)
        return fDevice;
      if (fDevice.isUsbHub())
      {
        fDevice = findDevice((UsbHub) fDevice, vendorId, productId);
        if (fDevice != null)
          return fDevice;
      }
    }
    return null;
   
  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method does nothing.*/

  public String receive (UsbInterface iface, byte endPoint)
  {

    UsbEndpoint endpoint = iface.getUsbEndpoint(endPoint);
    UsbPipe pipe = endpoint.getUsbPipe();

    byte[] data = null;
    int received = 0;
    try
    {
      pipe.open();
      data = new byte[64];
      received = pipe.syncSubmit(data);
    }
    catch (UsbException ue)
    {
    }

    System.out.println(received + " bytes received");

    String result = "";
    while (received-- > 0)
    {
      if (received == 1)
        result = result + Byte.toString(data[received]);
      else
        result = result + ", " + Byte.toString(data[received]);
    }

    return result;

  }
  //***************************************************************************

  //***************************************************************************
  /** This method does nothing.*/

  public static void send ()
  {

  }
  //***************************************************************************

  //***************************************************************************
  /** This method does nothing.*/

  public static void stopDevice()
  {

    System.out.println("Exiting");

    LibUsb.close(handle);
    LibUsb.exit(context);

    context = null;
    device = null;
    descriptor = null;
    handle = null;
    uConfiguration = null;
    uDevice = null;
    hub = null;
    iface = null;
    services = null;

  }
  //***************************************************************************

  //***************************************************************************
  /** This method tests for, and confirms access, to the TrueRNG device.*/

  public static boolean testClaim ()
  {

    boolean startedOK = false;                          // assume it will fail.

    // Initialize a context
    context = new Context();
    int result = LibUsb.init(context);
    if (result != LibUsb.SUCCESS)
    {
      throw new LibUsbException("Unable to initialize USB context", result);
    }
    else
      System.out.println("USB context initialized.");

    // Open test device
    handle = LibUsb.openDeviceWithVidPid(context, VENDOR_ID, PRODUCT_ID);
    if (handle == null)
    {
      System.err.println("Could not open TrueRNG USB device.");
      System.exit(1);
    }
    else
    {
      descriptor = new DeviceDescriptor();
      device = LibUsb.getDevice(handle);
      int dummy = LibUsb.getDeviceDescriptor(device, descriptor);
      System.out.println("USB identified and opened.");
      System.out.println ("Vendor ID: " + toHex(descriptor.idVendor()));
      System.out.println ("Product ID: " + toHex(descriptor.idProduct()));
      try
      {
        hub = services.getRootUsbHub();
      }
      catch (UsbException ue)
      {
        System.out.println("Could not identify hub in TrueRNG.testDevice.");
      }
      finally
      {
        uDevice = findDevice(hub, VENDOR_ID, PRODUCT_ID);
        System.out.println(uDevice == null? "bad uDevice": "");
        uConfiguration = uDevice.getActiveUsbConfiguration();
        System.out.println(uConfiguration == null? "bad uConfiguration": "");
        iface = uConfiguration.getUsbInterface((byte) 1);
        System.out.println(iface == null? "bad iface": "");
      }
    }

    // Start event handling thread
    EventHandlingThread thread = new EventHandlingThread();
    thread.start();

    // Attempt to reset the device
    System.out.println ((LibUsb.resetDevice(handle) == 0) ?
                        "TrueRNG USB device reset success":
                        "TrueRNG USB device reset failed");

    // Prepare the TrueRNG interface
    result = LibUsb.setConfiguration(handle, INTERFACE_1);
    if (result != LibUsb.SUCCESS)
    {
      System.err.println("Cannot set/reset TrueRNG USB device configuration.");
stopDevice();
      System.exit(1);
    }
    else
      System.out.println("USB set/reset.");

    // Claim the TrueRNG interface
    result = LibUsb.claimInterface(handle, INTERFACE_1);
    if (result != LibUsb.SUCCESS)
    {
      System.err.println("Cannot claim TrueRNG USB device interface.");
stopDevice();
      System.exit(1);
    }
    else
      System.out.println("USB interface claimed.");

    thread.abort();
    try
    {
      thread.join();
    }
    catch (InterruptedException ie)
    {
    }

    if (result == LibUsb.SUCCESS)
    {
      System.out.println("TrueRNG USB device started successfully.");
      startedOK = true;
    }
    
stopDevice();
return true;

//    return startedOK;

  }
  //***************************************************************************

  //***************************************************************************
  /** This method does nothing.*/

  public static String toHex(short shortArgument)
  {

    return Integer.toHexString(shortArgument & 0xffff);

  }
  //***************************************************************************

  //***************************************************************************
  /** This method does nothing.*/

  //public static void main (String [] args)
  //{

  //  System.out.println("Stand-alone diagnostic of the TrueRNG USB device.");

  //  dumpDevices();
  //  testDevice();

  //  System.exit (0);

  //} // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method does nothing.*/

  public boolean template ()
  {

    return false;

  } // End method.
  //***************************************************************************

}
