//
//*****************************************************************************
//*                                                                           *
//* COPYRIGHT (c) Maelzel, Inc.,                                              *
//*                                                                           *
//*          Redistribution and use of this code and the algorithms herein    *
//*          contained, with or without modification, is permitted provided   *
//*          that the following conditions are met:                           *
//*          * Redistribution of this code and the algorithms herein          *
//*            contained must retain the above copyright notice, this list    *
//*            of conditions, and the following disclaimer.                   *
//*          * Any use of this code and the algorithms herein contained as    *
//*            text or in binary form must attribute, in a reasonable         *
//*            manner, the provenance and authorship of this code as          *
//*            shown in the copyright above.                                  *
//*                                                                           *
//* DISCLAIMER This code and the algorithms herein contained are provided     *
//*            by the developer ''as is'' without any expressed or implied    *
//*            warranties, including, but not limited to, fitness for any     *
//*            particular purpose or any consequential damage that may        *
//*            occur as a consequence of use or association with this code    *
//*            or the algorithms herein contained.                            *
//*                                                                           *
//* TOP      Clans.java                                                       *
//* CLASS    ModelReports.java. (Model reports)                               *
//* AUTHOR   Ovi Chris Rouly                                                  *
//*                                                                           *
//* DATE     06/06/2015                                                       *
//* REVISION X.0.1.3                                                          *
//* BYTES    00,000                                                           *
//* TIME     13:32                                                            *
//* PURPOSE  To facilitate the run- and post-runtime capture of disk data.    *
//* METHOD   Each instance of a class is a stand-alone code object.           *
//* DETAILS  A Java document exists that outlines class operation.  Other     *
//*          documentation in the form of programmer notes apply.             *
//* ENVIRON  Developed on Intel Core-2 Quad Q8200 2.33 GHz CPU, Nvidia-based  *
//*          PCI-X graphics acceleration.  The OS was MS Windows 7 Ultimate.  *
//*          Java IDE was JGrasp version 2.0.0_08.  The Java compiler was     *
//*          version 1.8.0.  Verification tests were run in Java JRE          *
//*          build 1.8.0-b132 with client VM build 14.3-b01.                  *
//*                                                                           *
//*****************************************************************************
//

package root.writers;                                        // home directory.

import java.io.BufferedReader;                                // Java language.
import java.io.BufferedWriter;                                //         ditto.
import java.io.File;                                          //         ditto.
import java.io.FileInputStream;                               //         ditto.
import java.io.FileNotFoundException;                         //         ditto.
import java.io.FileWriter;                                    //         ditto.
import java.io.InputStreamReader;                             //         ditto.
import java.io.IOException;                                   //         ditto.
import java.util.Calendar;                                    //         ditto.
import java.util.ArrayList;                                   //         ditto.
import java.util.Arrays;                                      //         ditto.
import java.util.Collections;                                 //         ditto.
import java.util.Comparator;                                  //         ditto.
import java.util.GregorianCalendar;                           //         ditto.

import root.ModelConstraints;                              // program routines.

//
//*****************************************************************************
// Model Reports

public class ModelReports implements ModelConstraints
{

  //
  //***************************************************************************
  // Class constants, properties, and objects.

  // Constants.
  final String DAILY_REPORT_TITLE = "DailyReport.txt";   // common report name.
  final String DAILY_REPORT = "DailyStatistics";            // collated report.
  final String REAPER_REPORT_TITLE = "ReaperReport.txt"; // common report name.
  final String REAPER_REPORT = "BoundReaperReport";         // collated report.
  final String SOCIAL_REPORT = "SocialNetworkReport";       // collated report.
  final String BREEDERS_REPORT = "BaselineBreedingReport";// all breeders rprt.
  final String BREEDERS_REPORT_TYPE_1 = "PromicuousBreedingReport"; // segment.
  final String BREEDERS_REPORT_TYPE_2 = "MixedAltPromBreedingReport";//segment.
  final String BREEDERS_REPORT_TYPE_3 = "AltruisticBreedingReport"; // segment.
  final String ALLIANCE_REPORT = "BaselineAllianceReport";// all alliance rprt.
  final String ALLIANCE_REPORT_TYPE_1 = "PromicuousAllianceReport"; // segment.
  final String ALLIANCE_REPORT_TYPE_2 = "MixedAltPromAllianceReport";//segment.
  final String ALLIANCE_REPORT_TYPE_3 = "AltruisticAllianceReport"; // segment.

  final String q = "\"";   // quotation mark.
  final String lf = "\n";  // line feed.
  final String cr = "\r";  // carriage return.
  final String tab = "\t"; // tab.

  // Variables.
  private String sBuffer = new String("");       // the output buffered string.
  private String dataFile = new String("");     // the name of the output file.
  private static String dataFolderPath = "";        // path to the data folder.
  private String masterFileName = new String("");   // the current master file.
  private static int lastEpoch = 0;  // last epoch recorded in the Reaper file.

  // Objects.
  private BufferedWriter bWriter = null;             // this is an output file.
  private FileWriter fWriter = null;                 // this is an output file.
  private File f = null;                             // this is an output file.

  //***************************************************************************
  /** This is the default Model Report class constructor used for access to
      file manipulation tools resident in the Class.*/

  public ModelReports ()
  {

    dataFolderPath = findDataRepositoryPath ();     // path to the data folder.

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This is the Model Report class constructor used for periodic model
      report making.*/

  public ModelReports (String fileName)
  {

    this.dataFile = fileName;

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method appends a new record onto the output string.*/

  public void accumulate (String stringFragment)
  {

    sBuffer = sBuffer.concat( stringFragment );

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns a String built from an arrayList where the data
      in the arrayList is concatenated with the supplied delimiter.*/

  public String arrayListToDelimitedString (ArrayList <String> list,
                                            String delimiter)
  {

    String dummyString = "";

    // rebuild the String record with the new data.
    dummyString = list.get (0);
    for (int j = 1; j < list.size(); j++)
    {
      dummyString = dummyString + delimiter + list.get (j);
    }

    return dummyString;

  }
  // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method assembles the contents of the files listed in the listFiles
      parameter into a single, new file and deposit it into the data folder.*/

  public void assembleFiles (ArrayList <File> listFiles, String fileName)
  {

    System.out.println ("Collating " + fileName + ".");

    String line = new String("");

    open (fileName); // using a master fileName provided, open file for output.

    bWriter = new BufferedWriter(this.fWriter);
    FileInputStream fis = null;        // the input file(s) based on listFiles.
    BufferedReader bReader = null;                // the input stream buffered.

    for (File file : listFiles)
    {

      try
      {
        fis = new FileInputStream (file);
        bReader = new BufferedReader (new InputStreamReader(fis));
        while ((line = bReader.readLine()) != null)
        {
          bWriter.write(line);
          bWriter.newLine();
        }
      }
      catch (FileNotFoundException fnfe)
      {
        System.out.println("ERROR: ModelReport.assembleFiles fnfe 1.");
      }
      catch (IOException ioe)
      {
        System.out.println("ERROR: ModelReport.assembleFiles ioe 1.");
      }

      try
      {
        fis.close();                                         // close the file.
        bReader.close();                                   // close the buffer.
      }
      catch (FileNotFoundException fnfe)
      {
        System.out.println("ERROR: ModelReport.assembleFiles fnfe 2.");
      }
      catch (IOException ioe)
      {
        System.out.println("ERROR: ModelReport.assembleFiles ioe 2.");
      }

    }

    try
    {
      bWriter.close();                            // close the buffered writer.
      close (this.fWriter);                           // close the file writer.
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.assembleFiles fnfe 3.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.assembleFiles ioe 3.");
    }

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method closes the output file.*/

  public void close ()
  {

    try
    {

      try (BufferedWriter bWriter = new BufferedWriter(this.fWriter))
      {
        bWriter.write(sBuffer);            // write the capture string to disk.
        bWriter.close();                          // close the buffered writer.
      } // write the capture string to disk.

      this.fWriter.close();                                  // close the file.

    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.close() fnf.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.close() ioe.");
    }

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method attempts to close the FileWriter parameter.*/

  public void close (FileWriter fileWriter)
  {

    try
    {
      fileWriter.close();                                    // close the file.
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.close(fW) fnf.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.close(fW) ioe.");
    }

  }
  // End method.
  //***************************************************************************

  //***************************************************************************
  /** WARNING - Long method.
      This method searches through the Reaper data to collate records.  The
      records collated are related to the Cause Of Death of the homonid.*/

  public void codHistory ()
  {

    System.out.println("Cause-of-Death records being assembled.");

    String line = new String("");              // a line of data from the file.
    String [] lineArray = new String [31];    // struct to hold delimited data.
    int pointer = 0;         // points to the current cell in the reaper array.
    Integer maxCount = recordCount ();      // the number of records in Reaper.
    String [] reaperArray = new String [maxCount];  // local manipulable array.

    String fileName = getCurrentDateAndTime() + ("DeathHistory.dat");
    open (fileName);       // create a target (geneology) fileWriter (fWriter).
    this.bWriter = new BufferedWriter(this.fWriter);

    FileInputStream fis = null;         // the input file(s) based on fileName.
    BufferedReader bReader = null;                // the input stream buffered.
    File inputFile = new File (dataFolderPath + File.separator +
                               this.masterFileName);

    try
    {
      fis = new FileInputStream (inputFile);
      bReader = new BufferedReader (new InputStreamReader(fis));
      while ((line = bReader.readLine()) != null)
      {
        reaperArray[pointer++] = line;
      }
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.codHistory fnfe 1.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.codHistory ioe 1.");
    }

    fis = null;                       // return memory to the operating system.
    bReader = null;                   // return memory to the operating system.

    // ------------------------------------------------------------------------

    String [] copyArray = new String [11];            // a local working array.
    for (pointer = 0; pointer < reaperArray.length; pointer++)
    {
      lineArray = reaperArray[pointer].split(",");   // get next line in array.
      copyArray [0] = lineArray[1];       // epoch.

      copyArray [1] = (lineArray[23].equals("Male") ? "1" : "0");
      copyArray [2] = lineArray[22];      // age.

      copyArray [3]  = "0";               // initialize as false.
      copyArray [4]  = "0";               // initialize as false.
      copyArray [5]  = "0";               // initialize as false.
      copyArray [6]  = "0";               // initialize as false.
      copyArray [7]  = "0";               // initialize as false.
      copyArray [8]  = "0";               // initialize as false.
      copyArray [9]  = "0";               // initialize as false.
      copyArray [10] = "0";               // initialize as false.

      if (lineArray[30].equals("Infant mortality"))
        copyArray [3]  = "1";             // infant mortality.
      if (lineArray[30].equals("Catastrophy"))
        copyArray [4]  = "1";             // catastrophy.
      if (lineArray[30].equals("Starvation"))
        copyArray [5]  = "1";             // starvation.
      if (lineArray[30].equals("Dehydration"))
        copyArray [6]  = "1";             // dehydration.
      if (lineArray[30].equals("Age"))
        copyArray [7]  = "1";             // age.
      if (lineArray[30].equals("Orphaned"))
        copyArray [8]  = "1";             // orphaned.
      if (lineArray[30].equals("Predation"))
        copyArray [9]  = "1";             // predation.
      if (lineArray[30].equals("Drowning"))
        copyArray [10] = "1";             // drowning.

      reaperArray [pointer] = restoreString (copyArray);
    }
    lineArray = null;                 // return memory to the operating system.
    copyArray = null;                 // return memory to the operating system.

    // ------------------------------------------------------------------------

    try
    {
      for (pointer = 0; pointer < reaperArray.length; pointer++)
      {
        bWriter.write(reaperArray[pointer]);
        bWriter.newLine();
      }
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.codHistory fnfe 2.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.codHistory ioe 2.");
    }

    try
    {
      bWriter.close();                            // close the buffered writer.
      close (this.fWriter);                           // close the file writer.
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.codHistory ioe 3.");
    }

    reaperArray = null;               // return memory to the operating system.

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method directs the collation (consolidation of possibly several)
      reports.  If there are 1 or more, a single consolidated file dated and
      time by wall-clock time will be created in the data folder and will
      be named according to constant conventions.*/

  public void collateReports (int reportType)
  {

    String commonTitle = "";     // the common report name used during runtime.
    String collatedTitle = "";     // the collated (bound) version of the file.
    switch (reportType)
    {
      case 0: commonTitle = REAPER_REPORT_TITLE;
              collatedTitle = REAPER_REPORT;
              break;
      case 1: commonTitle = DAILY_REPORT_TITLE;
              collatedTitle = DAILY_REPORT;
              break;
      default: System.out.println("ERROR mode: ModelReports.collateReports");
               System.exit(0);
               break;
    }

    String fileName = getCurrentDateAndTime() + collatedTitle + ".dat";
    this.masterFileName = fileName;
    File file = new File (this.masterFileName);

    // collate a list of files in the data folder.
    ArrayList <File> listFiles = listFiles (dataFolderPath, commonTitle);

    // if there are 1 or more files then...
    int count = listFiles.size();
    if (count != 0)
    {
      // concatenate listFiles contents into a single file.
      assembleFiles(listFiles, fileName);
    }

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** WARNING - Long method.
      This method isolates four fields in the master reaper file.  These are
      are the self, mother, father, and sex of self.  Then, it deposits a new
      file constructed of those data into the data folder path.  The output is
      a Pajek and UCINet readable social network product.*/


     /* OUTPUT EXAMPLE for SocialNetworkReport.net:

      *Vertices 3
      1 bc RGBFF0000 ic RGBFF0000
      2 bc RGB0000FF ic RGB0000FF
      3 bc RGBFF00FF ic RGBFF00FF
      *Arcs
      3 1 1 c RGB000000
      3 2 1 c RGB000000

    */

  public void collateSocialNets ()
  {

    System.out.println("Geneological networks segregation in progress.");

    final String VERTICES = "*Vertices ";                 // a required header.
    final String ARCS = "*Arcs";                          // a required header.
    final String MALE = "RGB0000FF";                           // this is blue.
    final String FEMALE = "RGBFF0000";                          // this is red.
    final String BLACK = "RGB000000";                         // this is black.
    final String EDGE = " 1 c " + BLACK;    // this black edge has thickness 1.

    String line = new String("");              // a line of data from the file.
    String [] lineArray = new String [31];    // struct to hold delimited data.
    String sex = "";                        // from cell 23, the hominid's sex.

    Integer maxCount = recordCount ();      // the number of records in Reaper.
    String [] reaperArray = new String [maxCount];  // local manipulable array.

    String fileName = getCurrentDateAndTime() + SOCIAL_REPORT + ".net";
    open (fileName);       // create a target (geneology) fileWriter (fWriter).
    bWriter = new BufferedWriter(this.fWriter);

    FileInputStream fis = null;         // the input file(s) based on fileName.
    BufferedReader bReader = null;                // the input stream buffered.
    File inputFile = new File (dataFolderPath + File.separator +
                               this.masterFileName);

    // first, read reaper file into local manipulable array and process.
    int inner = 0;                    // pointer to the current record scanner.
    try
    {
      fis = new FileInputStream (inputFile);
      bReader = new BufferedReader (new InputStreamReader(fis));
      while ((line = bReader.readLine()) != null)
      {
        lineArray = line.split(",");             // substitute 0 for max count.
        if (lineArray[16].equals("0")) lineArray[16] =
                                       new Integer( maxCount + 1 ).toString();
        if (lineArray[17].equals("0")) lineArray[17] =
                                       new Integer( maxCount + 1 ).toString();
        reaperArray [inner++] = restoreString (lineArray);
      }
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.collateSocialNets fnfe 1.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.collateSocialNets ioe 1.");
    }

    fis = null;                       // return memory to the operating system.
    bReader = null;                   // return memory to the operating system.

    // second, replace all vertex labels starting with last one.
    int past = 0;                                         // go back into past.
    String self = "";
    Integer recordNr = new Integer (maxCount.toString());     // record number.
    for (int outer = reaperArray.length - 1; outer >= 0; outer--)
    {
      lineArray = reaperArray[outer].split(",");         // breakout the array.
      past = Integer.parseInt(lineArray[1]) - 10900;     // 30 years into past.
      self = lineArray[15];                        // take self as the example.
      lineArray[15] = recordNr.toString();  // replace symbol with the last ID.
      reaperArray [outer] = restoreString (lineArray);    // replace the array.
      inner = reaperArray.length;   // reset to the end of the array each time.
      while (( --inner >= 0 ) && (past < Integer.parseInt(lineArray[1])))
      {
        lineArray = reaperArray[inner].split(",");       // breakout the array.
        if (lineArray[16].equals (self)) lineArray[16] = recordNr.toString();
        if (lineArray[17].equals (self)) lineArray[17] = recordNr.toString();
        reaperArray [inner] = restoreString (lineArray);
      }
      recordNr--;                          // point to the next smaller symbol.
      //System.out.println (outer); // DEBUG
    }

    // third, enumerate the vertices and color them.
    recordNr = new Integer (1);             // the record number under process.
    try
    {
      bWriter.write(VERTICES +  new Integer( maxCount + 1 ).toString());
      bWriter.newLine();

      for (int outer = 0; outer < maxCount.intValue(); outer++)
      {
        lineArray = reaperArray[outer].split(",");       // breakout the array.
        sex = ( lineArray[23].equals ("Male") ? MALE : FEMALE );  // set color.
        line = recordNr.toString() + " " + q + q;            // construct line.
        line = line + " bc " + sex + " ic " + sex;                 // set line.
        recordNr++;                                       // increment counter.
        bWriter.write(line);                         // write the line to file.
        bWriter.newLine();                                     // add new line.
      }
      line = recordNr.toString() + " " + q + q;              // construct line.
      line = line + " bc " + BLACK + " ic " + BLACK;               // set line.
      bWriter.write(line);                           // write the line to file.
      bWriter.newLine();                                       // add new line.
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.collateSocialNets fnfe 2.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.collateSocialNets ioe 2.");
    }

    // fourth, the spanning edges need to be enumerated and colored.
    String mother = "";
    String father = "";
    try
    {
      bWriter.write(ARCS);
      bWriter.newLine();

      for (int outer = 0; outer < maxCount.intValue(); outer++)
      {
        lineArray = reaperArray[outer].split(",");       // breakout the array.
        self = lineArray[15]; mother = lineArray[16]; father = lineArray[17];
        line = self + " " + mother + EDGE;
        bWriter.write(line);
        bWriter.newLine();
        line = self + " " + father + EDGE;
        bWriter.write(line);
        bWriter.newLine();
      }
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.collateSocialNets fnfe 3.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.collateSocialNets ioe 3.");
    }

    try
    {
      bWriter.close();                            // close the buffered writer.
      close (this.fWriter);                           // close the file writer.
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.collateSocialNets ioe 4.");
    }

    reaperArray = null;               // return memory to the operating system.

    System.out.println("Network segregation complete.");

  }
  // End method.
  //***************************************************************************

  //***************************************************************************
  /** WARNING - Long method.
      This method searches through Daily Statistics data to collate records.
      The records collated are the year-end statements of the birth, death,
      and related records for the overall Sim.*/

  public void collateYearlyStats ()
  {

    System.out.println("Yearly statistic records being assembled.");

    String line = new String("");              // a line of data from the file.
    String [] lineArray = new String [11];    // struct to hold delimited data.
    int pointer = 0;     // points to the current cell in the statistics array.
    Integer maxCount = recordCount ();       // the number of records in array.
    String [] statsArray = new String [maxCount];   // local manipulable array.

    String fileName = getCurrentDateAndTime() + ("YearlyStats.dat");
    open (fileName);       // create a target (geneology) fileWriter (fWriter).
    this.bWriter = new BufferedWriter(this.fWriter);

    FileInputStream fis = null;         // the input file(s) based on fileName.
    BufferedReader bReader = null;                // the input stream buffered.
    File inputFile = new File (dataFolderPath + File.separator +
    //"7-2-2015_1-28-26-pm_BoundReaperReport.dat");
                               this.masterFileName);

    try
    {
      fis = new FileInputStream (inputFile);
      bReader = new BufferedReader (new InputStreamReader(fis));
      while ((line = bReader.readLine()) != null)
      {
        statsArray[pointer++] = line;
      }
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.collateYearlyStats fnfe 1.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.collateYearlyStats ioe 1.");
    }

    fis = null;                       // return memory to the operating system.
    bReader = null;                   // return memory to the operating system.

    // ------------------------------------------------------------------------

    int test = 0;                           // a variable used in calculations.
    int lastBirthCount = 0;                // last yearly birth count recorded.
    int lastDeathCount = 0;                // last yearly death count recorded.
    String [] copyArray = new String [8];             // a local working array.
    int outputPntr = 0;             // pointer into the disk file output array.
    for (pointer = 0; pointer < statsArray.length; pointer++)
    {
      lineArray = statsArray[pointer].split(",");    // get next line in array.

      // find the last day of each year.
      if ((Integer.parseInt(lineArray[2]) % YEAR) == 0)
      {

        copyArray [0] = lineArray[1]; // at the end of this year.
        copyArray [1] = lineArray[4]; // these were alive.
        // ------------
        test = Integer.parseInt(lineArray[6]);
        if (test != lastDeathCount)
        {
          test = Integer.parseInt(lineArray[6]) - lastDeathCount;
          copyArray [2] = (new Integer(test)).toString();  // deaths this year.
          lastDeathCount = Integer.parseInt(lineArray[6]);
        }
        else
        {
          copyArray [2] = "0";                             // deaths this year.
          lastDeathCount = test;
        }
        // ------------
        copyArray [3] = lineArray[6]; // total all deaths now recorded.
        // ------------
        test = Integer.parseInt(lineArray[7]);
        if (test != lastBirthCount)
        {
          test = Integer.parseInt(lineArray[7]) - lastBirthCount;
          copyArray [4] = (new Integer(test)).toString();  // births this year.
          lastBirthCount = Integer.parseInt(lineArray[7]);
        }
        else
        {
          copyArray [4] = "0";                             // births this year.
          lastBirthCount = test;
        }
        // ------------
        copyArray [5] = lineArray[7];  // total all births now recorded.
        copyArray [6] = lineArray[8];  // pre-calculated annual birth rate.
        copyArray [7] = lineArray[9];  // pre-calculated infant mortality rate.

        statsArray [outputPntr++] = restoreString (copyArray);
      }
    }
    lineArray = null;                 // return memory to the operating system.
    copyArray = null;                 // return memory to the operating system.

    // ------------------------------------------------------------------------

    try
    {
      for (pointer = 0; pointer < outputPntr; pointer++)
      {
        bWriter.write(statsArray[pointer]);
        bWriter.newLine();
      }
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.collateYearlyStats fnfe 2.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.collateYearlyStats ioe 2.");
    }

    try
    {
      bWriter.close();                            // close the buffered writer.
      close (this.fWriter);                           // close the file writer.
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.collateYearlyStats ioe 3.");
    }

    statsArray = null;               // return memory to the operating system.

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** WARNING - Long method.
      This method searches through the Reaper data to collate records.  The
      record created is a statement of how many members of each sex died in
      a given year.*/

  public void deathAnnuals ()
  {

    System.out.println("Death annual records being assembled.");

    String line = new String("");              // a line of data from the file.
    String [] lineArray = new String [31];    // struct to hold delimited data.
    int pointer = 0;         // points to the current cell in the reaper array.

    Integer maxCount = recordCount ();      // the number of records in Reaper.
    String [] reaperArray = new String [maxCount];  // local manipulable array.

    String fileName = getCurrentDateAndTime() + ("DeathAnnuals.dat");
    open (fileName);       // create a target (geneology) fileWriter (fWriter).
    this.bWriter = new BufferedWriter(this.fWriter);

    FileInputStream fis = null;         // the input file(s) based on fileName.
    BufferedReader bReader = null;                // the input stream buffered.
    File inputFile = new File (dataFolderPath + File.separator +
    //"7-2-2015_1-28-26-pm_BoundReaperReport.dat");
                               this.masterFileName);

    try
    {
      fis = new FileInputStream (inputFile);
      bReader = new BufferedReader (new InputStreamReader(fis));
      while ((line = bReader.readLine()) != null)
      {
        reaperArray[pointer++] = line;
      }
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.deathAnnuals fnfe 1.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.deathAnnuals ioe 1.");
    }

    fis = null;                       // return memory to the operating system.
    bReader = null;                   // return memory to the operating system.

    // ------------------------------------------------------------------------

    String [] workArray = new String [3];       // short, 3-cell working array.
    try
    {
      int epoch = 0;                         // the current date being checked.
      int maleDeaths = 0;             // the count of male deaths on this date.
      int femaleDeaths = 0;         // the count of female deaths on this date.

      // test if the reaper actually has recorded any deaths yet.
      if (reaperArray.length == 0)
      {
        bWriter.write("no deaths to report");
        bWriter.newLine();                 // write single line to output file.
      }
      else
      {
        pointer = 0;           // points to the first cell in the reaper array.
        lineArray = reaperArray[pointer].split(",");// parse the line in array.
        while (pointer < reaperArray.length)
        {

          do
          {
            epoch = Integer.parseInt(lineArray[1]);    // get the working date.
            do
            {

              if (lineArray[23].equals("Male"))
                maleDeaths++;              // if is male death increment males.
              else
                femaleDeaths++;        // if is female death increment females.

               // increment pointer and test if still in Reaper array.
              if (++pointer < reaperArray.length)
                lineArray = reaperArray[pointer].split(",");//parse array line.

            } while (Integer.parseInt(lineArray[1]) == epoch &&
                     pointer < reaperArray.length);// work date & inside limit.
          } while ((epoch % YEAR) != 0  &&
                    pointer < reaperArray.length);  // until year end continue.

          int currentYear = (int) (epoch / YEAR);
          workArray[0] = new Integer(currentYear).toString(); // transfer vars.
          workArray[1] = new Integer(maleDeaths).toString();
          workArray[2] = new Integer(femaleDeaths).toString();

          maleDeaths = 0;       // reset the count of male deaths on this date.
          femaleDeaths = 0;   // reset the count of female deaths on this date.

          bWriter.write(restoreString (workArray));
          bWriter.newLine();
        }
      }
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.deathAnnuals fnfe 2.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.deathAnnuals ioe 2.");
    }

    try
    {
      bWriter.close();                            // close the buffered writer.
      close (this.fWriter);                           // close the file writer.
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.deathAnnuals ioe 3.");
    }

    reaperArray = null;               // return memory to the operating system.
    workArray = null;                 // return memory to the operating system.

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns the path to the data folder.*/

  public String findDataRepositoryPath ()
  {

    String dataFolderPath = "";                     // path to the data folder.

    try
    {
      File sample = new File(BIN_FOLDER + DATA_FOLDER);                            // create.
      dataFolderPath = sample.getCanonicalPath(); // path to the "data" folder.
      sample = null;
    }
    catch (IOException ex)
    {
      System.out.println("ERROR: ModelReport.findDataRepositoryPath ioe.");
    }

    return dataFolderPath;

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns a Boolean that answers the question does the indi-
      vidual in the reaperArray have a positive altruistic phenocopy trait.*/

  public boolean getAlTrait (String individual, String [] reaperArray,
                             int today, int genePointer)
  {

    int THIRTY_YEARS = 10920;                // 13 months * 28 days * 30 years.
    int MINIMUM_GENE_VALUE_FOR_PHENOCOPY = 128;        // currently set to 128.

    //  find the individual in the reaperArray.  Adam & Eves will be missed.
    String [] lineArray = new String [31];    // struct to hold delimited data.

    int epoch = 0;                                  // the date of this record.
    int futureBreak = THIRTY_YEARS + today;  // future date 30 years in future.
    int cell = 0; // pointer to list cells beginning now to 30 years in future.

    boolean found = false;
    do
    {

      lineArray = reaperArray[cell].split(",");          // breakout the array.
      epoch = Integer.parseInt(lineArray[1]);  // get the (cell 1) current day.

      // test if individual (cell 15) is the hominid in the current record.
      if (individual.equals (lineArray[15]))
      {
        found = true;
      }

    // exit if found, testint beyond last cell, or 30 years into the future.
    } while ( !found &&
              (++cell < reaperArray.length) &&
              (epoch < futureBreak) );

    // isolate the gene value and return the answer to the question.
    // NOTE: by default all Adam & Eve generation agents will be FALSE!
    if (found)
    {
      found = false;
      int value = Integer.parseInt(lineArray[genePointer]); // gene under test.
      if (value >= MINIMUM_GENE_VALUE_FOR_PHENOCOPY)
      {
        found = true;
      }
    }

    lineArray = null;             // return the memory to the operating system.

    return found;

  }
  // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns a formatted string built of the date and time.*/

  public String getCurrentDateAndTime ()
  {

    int day, month, year = 0;
    int second, minute, hour = 0;
    int am_pm = 0;
    GregorianCalendar date = new GregorianCalendar();

    day = date.get(Calendar.DAY_OF_MONTH);
    month = date.get(Calendar.MONTH);
    year = date.get(Calendar.YEAR);

    second = date.get(Calendar.SECOND);
    minute = date.get(Calendar.MINUTE);
    hour = date.get(Calendar.HOUR);

    am_pm = date.get(Calendar.AM_PM);

    return day + "-" + (month + 1) + "-" + year + "_" +
           hour + "-" + minute + "-" + second + "-" +
           (am_pm == 0? "am" : "pm") + "_";

  }
  // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns a list of the files in the data folder.*/

  public ArrayList <File> listFiles (String dirName, String fileSubtitle)
  {

    ArrayList <File> fileList = new ArrayList <File> ();

    File directory = new File (dirName);
    File [] fList = directory.listFiles(); // provides default alphabetization.

    Arrays.sort (fList, new Comparator <File> () {
      public int compare (File f1, File f2)
      {
        return Long.valueOf(f1.lastModified()).compareTo(f2.lastModified());
      }
    } );

    for (File file : fList)
    {
      if (file.toString().contains((CharSequence) fileSubtitle))
      {
        fileList.add(file);
      }
    }

    return fileList;

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method opens the output file using the data parameter as part of
      the new file name.*/

  public void open (int date)
  {

    // construct a fileName using sim epoch date and user's dataFile name.
    String simDate = Integer.toString ((date / YEAR) + 1); // simulation epoch.
    String fileName = "year=" + simDate + "_" + this.dataFile;

    // construct the full path to the output data folder including fileName.
    dataFolderPath = findDataRepositoryPath ();     // path to the data folder.
    String fullPath = dataFolderPath + File.separator + fileName;

    openHelper (fullPath);

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method opens the output file using the fileName parameter.*/

  public void open (String fileName)
  {

    // construct the full path to the output data folder including fileName.
    dataFolderPath = findDataRepositoryPath ();     // path to the data folder.
    String fullPath = dataFolderPath + File.separator + fileName;

    openHelper (fullPath);

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method opens the output file using the data parameter as part of
      the new file name.*/

  public void openHelper (String fullPath)
  {

    try
    {
      this.f = new File (fullPath);
      boolean dummy = f.createNewFile();
      this.fWriter = new FileWriter (f);
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.openHelper fnfe.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.openHelper ioe.");
    }

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** WARNING - Long method.
      This method identifies and returns a list of matings by year.*/

  public ArrayList <String> matingsList (int geneValue)
  {

    ArrayList <String> mList = new ArrayList <String>();   // the matings list.

    String line = new String("");              // a line of data from the file.
    String [] lineArray = new String [31];    // struct to hold delimited data.
    String sex = "";                        // from cell 23, the hominid's sex.

    Integer maxCount = recordCount ();      // the number of records in Reaper.
    String [] reaperArray = new String [maxCount];  // local manipulable array.

    FileInputStream fis = null;         // the input file(s) based on fileName.
    BufferedReader bReader = null;                // the input stream buffered.
    File inputFile = new File (dataFolderPath + File.separator +
                               this.masterFileName);

    // ------------------------------------------------------------------------
    // first, read reaper file into local manipulable array.

    int pntr = 0;                     // pointer to the current record scanner.
    try
    {
      fis = new FileInputStream (inputFile);
      bReader = new BufferedReader (new InputStreamReader(fis));
      while ((line = bReader.readLine()) != null)
      {
        lineArray = line.split(","); // read line, parse it, split it, save it.
        reaperArray [pntr++] = restoreString (lineArray);
      }
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.matingsList fnfe 1.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.matingsList ioe 1.");
    }

    fis = null;                       // return memory to the operating system.
    bReader = null;                   // return memory to the operating system.
    // ------------------------------------------------------------------------

    // ------------------------------------------------------------------------
    // second, find the mom and dad of self (and their AlTraits).  Bundle the
    // traits together with mom and dad's IDs, respectively.  Concatenate mom,
    // dad, and the year the mating (self conception) occurred as a string.
    // Then, put the mating record into the mating records arraylist.

    String idSelf = "0";                                           // temp var.
    String idMom = "0";                                            // temp var.
    String idDad = "0";                                            // temp var.
    int   today = 0;                                               // temp var.
    int    year = 0;                                               // temp var.
    String alTrait = "";   // T or F symbol for person having altruistic trait.
    String matingRecord = ""; // concatenated year, mom & dad, and alTrait str.
    String phenoType = "";    // phenotype  1=bothNoAlt, 2=eitherIs, 3=bothAlt.

    for (pntr = 0; pntr < reaperArray.length; pntr++)
    {

      lineArray = reaperArray[pntr].split(",");          // breakout the array.

      today = Integer.parseInt(lineArray[1]);               // the current day.
      year = (int) today / YEAR ;                            // the birth year.
      idSelf = lineArray[15];                      // take self as the example.
      idMom = lineArray[16];                        // this is mom of the self.
      idDad = lineArray[17];                        // this is dad of the self.

      alTrait = (getAlTrait(
                 idMom, reaperArray, today, geneValue) ? "T" : "F" );   // mom?
      idMom = alTrait + idMom;
      alTrait = (getAlTrait(
                 idDad, reaperArray, today, geneValue) ? "T" : "F" );   // dad?
      idDad = alTrait + idDad;

      // altruistic phenotype  1 = bothNoAlt, 2 = eitherIs, 3 = bothAlt.
      if (!idMom.substring(0,1).equals(idDad.substring(0,1)))
        phenoType = "2";
      else if (idMom.substring(0,1).equals("T") &&
               idDad.substring(0,1).equals("T"))
        phenoType = "3";
      else
        phenoType = "1";

      // output form is: year-T123-F321-phenoType.
      matingRecord = Integer.toString (year) + "-" +
                     idMom + "-" + idDad + "-" +
                     phenoType;        // make a record string with delimiters.
      mList.add (matingRecord); // add concatenated, delimited string to mList.

    }

    return mList;                       // return the mating records arraylist.

  }
  // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method sorts a list of matings and returns the list grouped by
      mated pair.*/

  public ArrayList <String> matingsSorter (ArrayList <String> list)
  {

    int REPRODUCTIVE_LIFESPAN = 30;

    ArrayList <String> sortedList = new ArrayList <String> ();
    ArrayList <String> dummyList = new ArrayList <String> ();

    int year = 0;                  // the year described in the current record.
    int lastMatingYear = 0; // the last year the mated pair are known to breed.
    int phenotype = 0;       // altruistic phenocopy of mom and dad under test.
    int breakoutYear = 0;   // a time in the future when the agent cannot mate.
    String matedPair = "";   // a breeding pair identified by AlTraits and IDs.
    String record = "";      // a record (sample) taken from the incoming list.
    String sample = "";      // a sample (record) taken from the incoming list.

    int i = 0;                                        // pointer into the list.
    int j = 0;                                        // pointer into the list.
    int index = 0;                           // pointer into the record itself.
    int count = 0;         // the number of times this breeding pair has mated.
    do
    {

      i = 0;                        // pointer to the first record in the list.
      j = 0;                      // pointer to the current record in the list.
      count = 0;                               // reset the occurrance counter.
      record = list.get(i);            // always pull from the top of the list.

      // input form is: year-T123-F321-phenoType.
      dummyList = parseStringToArrayList (record, "-");

      year = Integer.parseInt ( dummyList.get (0) );  // you need breakoutYear.
      matedPair = dummyList.get (1) + "-" + dummyList.get (2);    // mom & dad.
      phenotype = Integer.parseInt ( dummyList.get (3) ); // this pair alTrait.

      lastMatingYear = year;  // until disproved, this is the last mating year.
      breakoutYear = year + REPRODUCTIVE_LIFESPAN;   // search during lifespan.

      // find the matedPair in the list, count it, then remove its record.
      do
      {

        if (!list.isEmpty())
        {
          record = list.get (i++);

          // input form is: year-T123-F321-phenoType.
          dummyList = parseStringToArrayList (record, "-");
          year = Integer.parseInt ( dummyList.get (0) );  // get the year data.
          sample = dummyList.get (1) + "-" + dummyList.get (2);   // mom & dad.

          if (sample.equals (matedPair))
          {
            j = list.indexOf (record);    // in list what position is the data?
            count ++;    // increment the count that occurance has in the list.
            list.remove(j);                // remove the current data instance.
            lastMatingYear = year;             // this is the last mating year.
            i = j; // since you removed the current record restore the pointer.
          }
        }

      }
      while ((year < breakoutYear) && ! list.isEmpty() && i < list.size());

      // add to the list all mated pairs except pure Adam & Eve (F0) pairs.
      if (!matedPair.equals ("F0-F0"))
      {
        // output form is: year-T123-F321-count-phenoType.
        record = new Integer ( lastMatingYear ).toString () + "-" +
                 matedPair + "-" +
                 new Integer ( count ).toString () + "-" +
                 new Integer ( phenotype ).toString ();
        sortedList.add (record);

      }

    }
    while (! list.isEmpty());          // if the list is NOT empty do continue.

    dummyList = null;                     // return memory to operating system.

    return sortedList;

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This returns the file name needed for the altruists and alliance
      filter file writer.*/

  public String outfileHelper (int genotype, int phenotype)
  {

    String outputFileName = "dummyFileName";
    
    // get file names according to genotype.
    switch (genotype)
    {
      case 0:
        // for altruists select a report file name for desired phenocopy.
        switch (phenotype)
        {
          case 1:
            outputFileName =  BREEDERS_REPORT_TYPE_1;
            break;
          case 2:
            outputFileName =  BREEDERS_REPORT_TYPE_2;
            break;
          case 3:
            outputFileName =  BREEDERS_REPORT_TYPE_3;
            break;
          default:
            outputFileName =  BREEDERS_REPORT;
            break;
        }
        break;
      case 1:
        // for alliance select a report file name for desired phenocopy.
        switch (phenotype)
        {
          case 1:
            outputFileName =  ALLIANCE_REPORT_TYPE_1;
            break;
          case 2:
            outputFileName =  ALLIANCE_REPORT_TYPE_2;
            break;
          case 3:
            outputFileName =  ALLIANCE_REPORT_TYPE_3;
            break;
          default:
            outputFileName =  ALLIANCE_REPORT;
            break;
        }
        break;
      default:
        System.out.println ("Error Mode: in modelreports outfileHelper.");
        outputFileName =  "ErrorMode";
        break; // do nothing.
    }

    return outputFileName;

  }
  // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method binds mating and breeding reports.*/

  public void pairingsReport ()
  {

    System.out.println("Collating altruism and alliance phenocopy reports.");

    int OFFSET = 4;    // relative to gene nr the cell position is offset by 4.

    ArrayList <String> outputList = new ArrayList <String> ();  // output list.
    String outFile = "";  // output filename based on list phenocopy selection.
    int geneInTest = 0;             // temp variable for the gene being tested.
    String geneTestType = "";       // temp variable for the gene being tested.
    ArrayList <String> mList = null;       // create a list of matings by year.
    ArrayList <String> yearlyStats = null;                  // statistics list.

    // scan by genotype, first for  altruism=0 then for alliance=1.
    for (int genoType = 0; genoType < 2; genoType++)
    {
      switch (genoType)
      {
        case 0:
          geneInTest = SOCIAL_ALTRUISM_GENE;              // point to the gene.
          geneTestType = "SOCIAL_ALTRUISM_GENE";    // english version thereof.
          break;
        case 1:
          geneInTest = PAIR_BND_ALLIANCE_GENE;            // point to the gene.
          geneTestType = "PAIR_BND_ALLIANCE_GENE";  // english version thereof.
          break;
        default:
          System.out.println ("Error Mode: bad genotype pairingsReport");
          System.exit(0);
          break;
      }

      // REMEMBER ReaperArray access requires a pointer offset into genes.
      mList = matingsList(geneInTest + OFFSET);    // matings by gene and year.
      if (mList.size() == 0)
      {
        System.out.println("No " + geneTestType + " type found.");
      }
      else
      {
        mList = matingsSorter (mList);      // sort into groups of mated pairs.

        // print 1 of 4 lists based on form of phenotype selected.
        for (int phenoType = 0; phenoType <= 3; phenoType++)
        {

          outFile = outfileHelper (genoType, phenoType); // phenotypes by name.
          yearlyStats = yearlySorter(mList, phenoType);         // create list.

          // prepare an output file for writing.
          String fileName = getCurrentDateAndTime() + outFile + ".net";
          open (fileName); // create (geneology) fileWriter (fWriter).
          bWriter = new BufferedWriter(this.fWriter);

          // write to the output file.
          try
          {
            // loop through the data and write the records out to file.
            for (int i = 0; i < yearlyStats.size(); i++)
            {
              bWriter.write(yearlyStats.get(i));
              bWriter.newLine();
            }

            bWriter.close();                      // close the buffered writer.
          }
          catch (IOException ioe)
          {
            System.out.println("ERROR: ModelReport.pairingsReport ioe.");
          }
        } // end 1 of 4 tests for empty list.


      } // end outer test for empty list
    } // end genotype typing loop.

    mList = null;                     // return memory to the operating system.
    yearlyStats = null;               // return memory to the operating system.

  }
  // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns the compartmentalized result of splitting a string
      concatenated together around one or more delimiters of the same type.
      It will return an empty array list if the string supplied is the empty
      string.  It will return the string supplied in a single-cell array list
      if the delimiter specified is not found.  Else, it will return each
      portion of the delimited string as an ArrayList contain each piece of
      the delimited string provided.  It is immune to orphaned leading or
      trailing delimiters.*/

  public static ArrayList <String> parseStringToArrayList (String string,
                                                           String delimiter)
  {

    int FRONT = 0;

    ArrayList <String> result = new ArrayList <String> ();
    int delimiterLocation = 0;

    if ( string.isEmpty () )
    {
      ;// do nothing.
    }
    else if ( !string.contains ( delimiter ) )
    {
      result.add ( string );
    }
    else
    {

      // typical delimited string as: "year-mom-dad-count"
      String dummy = string.toString ();            // make a manipulable copy.

      while (dummy.contains(delimiter))
      {
        delimiterLocation = dummy.indexOf ( delimiter );     // find delimiter.
        if (FRONT != delimiterLocation)
        {
          result.add( dummy.substring( FRONT, delimiterLocation )); //add data.
        }
        dummy = dummy.substring (delimiterLocation + 1);     // shorten string.
      }

      if (!dummy.isEmpty())
      {
        result.add ( dummy );
      }

    }

    return result;

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns the count of records in the reaper file.*/

  public Integer recordCount ()
  {

    int count = 0;
    String line = new String("");              // a line of data from the file.
    String [] lineArray = new String [31];    // struct to hold delimited data.

    FileInputStream fis = null;         // the input file(s) based on fileName.
    BufferedReader bReader = null;                // the input stream buffered.
    File inputFile = new File (dataFolderPath + File.separator +
    //"7-2-2015_1-28-26-pm_BoundReaperReport.dat");
                               this.masterFileName);

    try
    {
      fis = new FileInputStream (inputFile);
      bReader = new BufferedReader (new InputStreamReader(fis));
      while ((line = bReader.readLine()) != null)
      {
        lineArray = line.split(",");             // substiture 0 for max count.
        lastEpoch = Integer.parseInt(lineArray [1]);  // side-effect of method.
        count++;
      }
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.recordCount fnfe.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.recordCount ioe 1.");
    }

    fis = null;                       // return memory to the operating system.
    bReader = null;                   // return memory to the operating system.

    return new Integer (count);

  }
  // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method closes the currently open file, clears the local string
      buffer recording data, and opens a fresh copy of the file using
      the data parameter as part of the new file name.*/

  public void restart (int date)
  {

    // based on the current defaults attempt to close the file.
    this.close ();

    // the output buffered string can now be restarted.
    sBuffer = new String("");

    // similarly, open another based on the defaults and the date.
    this.open (date);

  }
  // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns a string constructed of the comma delimited
      elements of the parameter array.*/

  public String restoreString (String [] array)
  {

    String result = "";

    int i = 0;
    do
    {
      result = result + array [i] + "," ;
    } while (++i < (array.length - 1));

    return result + array [i];

  }
  // End method.
  //***************************************************************************

  //***************************************************************************
  /** WARNING - Long method.
      This method searches through Daily Statistics data to collate records.
      The record created is the terrain productivity on the first day of the
      season.*/

  public void seasonalProductivity ()
  {

    System.out.println("Seasonal terrain productivity stats assembly.");

    String line = new String("");              // a line of data from the file.
    String [] lineArray = new String [11];    // struct to hold delimited data.
    int pointer = 0;     // points to the current cell in the statistics array.
    Integer maxCount = recordCount ();       // the number of records in array.
    String [] statsArray = new String [maxCount];   // local manipulable array.

    String fileName = getCurrentDateAndTime() + ("SeasonalStats.dat");
    open (fileName);       // create a target (geneology) fileWriter (fWriter).
    this.bWriter = new BufferedWriter(this.fWriter);

    FileInputStream fis = null;         // the input file(s) based on fileName.
    BufferedReader bReader = null;                // the input stream buffered.
    File inputFile = new File (dataFolderPath + File.separator +
    //"7-2-2015_1-28-26-pm_BoundReaperReport.dat");
                               this.masterFileName);

    try
    {
      fis = new FileInputStream (inputFile);
      bReader = new BufferedReader (new InputStreamReader(fis));
      while ((line = bReader.readLine()) != null)
      {
        statsArray[pointer++] = line;
      }
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.seasonalProductivity fnfe 1.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.seasonalProductivity ioe 1.");
    }

    fis = null;                       // return memory to the operating system.
    bReader = null;                   // return memory to the operating system.

    // ------------------------------------------------------------------------

    final int SPRING = 1;           // the defined first day(s) of each season.
    final int SUMMER = 91;
    final int FALL = 182;
    final int WINTER = 273;

    String [] copyArray = new String [3];             // a local working array.
    int outputPntr = 0;             // pointer into the disk file output array.
    for (pointer = 0; pointer < statsArray.length; pointer++)
    {
      lineArray = statsArray[pointer].split(",");    // get next line in array.

      // identify the first day of each season.
      int day = Integer.parseInt(lineArray[2]);
      if (day == SPRING || day == SUMMER || day == FALL || day == WINTER)
      {

        copyArray [0] = lineArray[1]; // this is the current year.

        if (Integer.parseInt(lineArray[2]) == SPRING)
          copyArray [1] = "SPRING";         // this is a season representation.
        else if (Integer.parseInt(lineArray[2]) == SUMMER)
          copyArray [1] = "SUMMER";         // this is a season representation.
        else if (Integer.parseInt(lineArray[2]) == FALL)
          copyArray [1] = "FALL";           // this is a season representation.
        else if (Integer.parseInt(lineArray[2]) == WINTER)
          copyArray [1] = "WINTER";         // this is a season representation.
        else
          System.out.println("ERROR: ModelReport.seasonalProductivity season");

        copyArray [2] = lineArray[3]; // the productivity of the terrain.

        statsArray [outputPntr++] = restoreString (copyArray);
      }
    }
    lineArray = null;                 // return memory to the operating system.
    copyArray = null;                 // return memory to the operating system.

    // ------------------------------------------------------------------------

    try
    {
      for (pointer = 0; pointer < outputPntr; pointer++)
      {
        bWriter.write(statsArray[pointer]);
        bWriter.newLine();
      }
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("ERROR: ModelReport.seasonalProductivity fnfe 2.");
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.seasonalProductivity ioe 2.");
    }

    try
    {
      bWriter.close();                            // close the buffered writer.
      close (this.fWriter);                           // close the file writer.
    }
    catch (IOException ioe)
    {
      System.out.println("ERROR: ModelReport.seasonalProductivity ioe 3.");
    }

    statsArray = null;               // return memory to the operating system.

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method returns an arraylist of Strings wherein each String in the
      list is a consecutive (by year) accounting of the breeding types that
      occurred in that given year.  Phenocopy allows the list building
      functionality to filter by expressive genetic type.*/

  public ArrayList <String> yearlySorter (ArrayList <String> matingRecords,
                                          int phenocopy)
  {

    int MAX_BREEDINGS = 7;   // the max number of effective breeding instances.
    String HYPHEN = "-";    // the delimiter between years, mom & dad, & count.
    String COMMA = ",";   // the delimiter between yearly breeding type totals.
    String MASKING_STRING = "yr,0,0,0,0,0,0,0"; // data struct 1, 2, ..., 6, 7.

    // intialize the breeding statistics data structure, this is the result.
    ArrayList <String> sortedList = new ArrayList <String> ();  // target list.

    // lastEpoch is a side-effect of the reaperArray recordCount method.
    for (int i = 0; i < (((int) (lastEpoch / YEAR)) + 1); i++)
    {
      sortedList.add(MASKING_STRING);    // put new record into the array list.
    }

    // using matingRecords update the breeding statistics data structure.
    String yrStr = "";                       // String form of the record year.
    int year = 0;                           // integer form of the record year.
    int breedingFreq = 0;              // the breeding frequency for this pair.
    int phenotype = 0;               // 1 of 3 AlTrait genetic types expressed.
    int typeCount = 0;    // times this type of breeding activity has occurred.
    ArrayList <String> dummyList = null;      // temporary array list variable.
    String dummyString = "";                      // temporary string variable.

    // this is an error trap for a simulation stopped to early.
    if (matingRecords.isEmpty())
    {
      sortedList.add("No non-pure seed group matings to report.");
      return sortedList;
    }

    // work down the master list translating breeding frequencies into Strings.
    // each record in the matingRecords is for ONE MATING, NOT A CLASS OF THEM.
    for (String record : matingRecords)
    {

      // take a String out of matingRecords String arrayList and recover data.
      // input form is: year-T123-F321-count-phenoType.
      dummyList = parseStringToArrayList (record, HYPHEN);
      yrStr = dummyList.get (0);                               // get the year.
      year = Integer.parseInt (yrStr);                     // convert the year.
      breedingFreq = Integer.parseInt (dummyList.get (3));    // get frequency.
      phenotype = Integer.parseInt (dummyList.get (4));   // get alTrait pCopy.

      // filter by phenocopy, if matches type or is 0=all then go ahead.
      if ((phenocopy == phenotype) || (phenocopy == 0))
      {
        // adjust breedingFreq pointer to accomodate data structure.
      //breedingFreq--;                      // data struct is 1, 2, ..., 6, 7.

        // using the year as the index into the target list and the
        // breedingFreq as the index into the Strings of sortedList,
        // increment the count of repeated pair-wise breeding occurances.
        dummyString = sortedList.get (year);
        dummyList = parseStringToArrayList (dummyString, COMMA);
        typeCount = Integer.parseInt (dummyList.get (breedingFreq));

        // output format is: yr,#,#,#,#,#,#,#
        dummyList.set (0, yrStr);        // update the record year in the list.
        dummyList.set (breedingFreq, (new Integer(++typeCount).toString()));

        // rebuild the String record with the new data.
        dummyString = arrayListToDelimitedString(dummyList, COMMA); // convert.

        // put the String back into the target list.
        sortedList.set(year, dummyString);
      }

    }

    // remove any unused lines in the list starting at the bottom (last)
    // entry, working your way upward (to the first), and stopping the
    // removal process with the first "used" line.
    boolean done = false;
    int pointer = sortedList.size() - 1;
    do
      if (sortedList.get (pointer).equals (MASKING_STRING))
        sortedList.remove(pointer);
      else
        done = true;
    while ((--pointer >= 0) && !done);

    // scan the list from the top (first entry) to the bottom and replace any
    // "yr" markers (in unused entries) with the current (missing) year.
    if (sortedList.size() > 0)
    {
      year = 0;
      do
      {
        dummyString = sortedList.get ( year );
        if ( dummyString.contains ( "yr" ) )
        {
          dummyList = parseStringToArrayList (dummyString, COMMA);
          dummyList.set ( 0, new Integer ( year ).toString () );
          dummyString = arrayListToDelimitedString(dummyList, COMMA);//convert.
        }
        sortedList.set ( year, dummyString );   // put the String back in list.
      }
      while ( ++year < sortedList.size () );
    }
    else
    {
      sortedList.add ("List of type " + phenocopy + " is empty.");
    }

    return sortedList;

  } // End method.
  //***************************************************************************

  //***************************************************************************
  /** This method does nothing.*/

  public void template ()
  {

    // add code here.

  }
  // End method.
  //***************************************************************************

}
